use mquery::query::{
    ops::{Arithmetic, Comparison, Logical},
    Metric, QueryExt, Scalar,
};

mod utils;

// Arbitrary queries to test whether they produce valid syntax
// The server will return an error if there is a syntax error and the test will fail

#[tokio::test]
async fn arithmetic() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .add(Metric::new("another_metric").label_ne("label", "value"))
        .mul(Scalar(0.1))
        .sub(Metric::new("third_metric"))
        .div(Scalar(0.5))
        .pow(Scalar(2.5));
    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn arithmetic_match_on() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .add(Metric::new("another_metric").label_ne("label", "value"))
        .mul(Scalar(0.1))
        .sub(Metric::new("third_metric"))
        .div(Scalar(0.5))
        .pow(Scalar(2.5))
        .match_on(["label"]);
    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn arithmetic_ignoring() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .add(Metric::new("another_metric").label_ne("label", "value"))
        .mul(Scalar(0.1))
        .sub(Metric::new("third_metric"))
        .div(Scalar(0.5))
        .pow(Scalar(2.5))
        .ignoring(["label"]);
    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn comparison() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .cmp_eq(Metric::new("another_metric").label_ne("label", "value"))
        .cmp_ne_bool(Scalar(0.1))
        .cmp_gt(Metric::new("third_metric"));

    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn comparison_match_on() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .cmp_eq(Metric::new("another_metric").label_ne("label", "value"))
        .cmp_ne_bool(Scalar(0.1))
        .cmp_gt(Metric::new("third_metric"))
        .match_on(["label"]);

    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn comparison_ignoring() {
    let query = Metric::new("test_metric")
        .label_eq("label", "value")
        .cmp_eq(Metric::new("another_metric").label_ne("label", "value"))
        .cmp_ne_bool(Scalar(0.1))
        .cmp_gt(Metric::new("third_metric"))
        .ignoring(["label"]);

    utils::send_query(query).await.unwrap();
}

#[tokio::test]
async fn logical_set() {
    let query = Metric::new("test_metric")
        .and(Metric::new("another_metric"))
        .or(Metric::new("third_metric"))
        .unless(Metric::new("fourth_metric"));
    utils::send_query(query).await.unwrap();
}

#[cfg(feature = "metricsql")]
#[tokio::test]
async fn keep_metric_names_mod() {
    let query = Metric::new("metric")
        .add(Metric::new("metric2"))
        .keep_metric_names();
    utils::send_query(query).await.unwrap();
}
